function [q_vek,qp_vek] = RTRAFO_TR_ges(p_vek,p_punkt_vek,a2,q_min,q_max)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% W. Weber, September 2023
% Beim RTRAFO fr den TR gibt es im Allgemeinen 2 Lsungsvektoren: Konfiguration A
% und Konfiguration B, bei mehreren vernnftigen Lsungen wird Konfiguration A bevorzugt.  
% Konfiguration A:  0<= q2 >= 180 (wobei der Gelenkarbeitsbereich zu beachten ist)  
% Es werden Lsungen im Bewegungsbereich gesucht und solche Lsungen vorzugsweise genommen,
% die keine Umorientierungen verursachen.
% Wenn im Startpunkt und Zielpunkt die gleiche Konfiguration genommen werden kann, dann ist eine 
% Umorientierung unwahrscheinlich
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   RTRAFO fr den TR-Roboter (siehe ausfhrliches Beispiel zu 
%   Master Mechatronik HFB, Lehrveranst. Motion Control (mc) (mc
%   oder BSR04 
%   W. Weber: September 2023
%   nderung; 23.08.2021
%
%   Eingabe: p_vek=[px;py;pz] : Gelenkkoordinaten p_vek(3,anz) anz: zeitl. Serie von p  
%   Eingabe: p_punkt_vek:  kart. Geschwindigkeitsvektor , anz: zeitl. Serie von p_punkt 
%   Hier ist py konstant 0 
%   
%   Ausgabe: q_vek=[q_vek(1,anz); q_vek(2,anz]
%    Ausgabe: qp_vek=[qp_vek(1,anz); qp_vek(2,anz]
%      
%
%   Die Transformation basiert auf folgenden D.-H-Parametern
%   d1=q1, d2=0, a1=0, a2=l2=0.5, alpha1=pi/2, alpha2=0;
%
%   
%   letzte nderung 21.09.23, W. Weber 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
global T_Ipo
eps_sing=0.00001;   % Grenzwert fr Singularitt bei q2=180

dm=size(p_vek); 
anz_schritte=dm(1,2);  % Anzahl der Spaltenvektoren im Abstand T_Ipo


q_vek=zeros(2,anz_schritte);
q_veka=zeros(2,anz_schritte);
q_vekb=zeros(2,anz_schritte);

qp_vek=zeros(2,anz_schritte);

% Analytischer RTRAFO mit im Allg. 2 Lsungen

% Betrag des Bewegungsbereiches
q_bereich=abs(q_max-q_min);

% Mitte des Bewegungsbereiches
q_mitte=q_min+q_bereich/2;

ja=120000; % nur Startwert fr Gteindex
jb=120000; % nur Startwert fr Gteindex

% beim Startpunkt i1=1 wird eine Lsung mit der weitesten Entfernung zu qmin,
% qmax gewhlt, es wird die Vektornorm verwendet

%xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
%lsungen a,b fr den Startpunkt/Zielpunkt, ohne Betrachtung
% des Konfigurationraumes


if anz_schritte==1  % hier fr  Aufruf , fr Pruef_Bahn_TR
  
   % disp('Gelenkwinkel q2 fr Startpunkt/Zielpunkt ohne Manipulation')
    q_2a=acos(-p_vek(1,1)/a2) % liefert Werte zwischen 0 und 180
    q_2b=-q_2a
          
      
   if q_2a > q_max(2,1) |  q_2a < q_min(2,1)
        q_2a=q_2a-sign(q_2a)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
   end
   
   if q_2b > q_max(2,1) |  q_2b < q_min(2,1)
        q_2b=q_2b-sign(q_2b)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
    end
    
    q_1a=p_vek(3,1)-a2*sin(q_2a);
    q_1b=p_vek(3,1)-a2*sin(q_2b);
    
    % ' Lsungen nach eventueller Manipulation'
  % q_1a 
  % q_1b
  % q_2b 
    
       if (q_1a <= q_max(1,1) & q_1a >= q_min(1,1)) & ...
         (q_2a <= q_max(2,1) & q_2a >= q_min(2,1))
        q_vek(:,1)=[q_1a;q_2a] % bevorzugt: Lsung im ersten Konfig.raum (Effektor zeigt nach rechsts) 
       elseif (q_1b <= q_max(1,1) & q_1a >= q_min(1,1)) & ...
         (q_2b <= q_max(2,1) & q_2b >= q_min(2,1))
        q_vek(:,1)=[q_1b;q_2ba]   
     
       else
           q_vek(:,1)=[10000;10000] % hier keine Lsung
       end
    end % anz_schritte==1 ,nur Startpunkt oder Zielpunkt

    
    % xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
    if anz_schritte > 1
        
    % nochmals Sttartpunkt
     % disp('Gelenkwinkel q2 fr Startpunkt ohne Manipulation')
    qSt_2a=acos(-p_vek(1,1)/a2); % liefert Werte zwischen 0 und 180
    qSt_2b=-qSt_2a;
          
      
   if qSt_2a > q_max(2,1) |  qSt_2a < q_min(2,1)
        qSt_2a=qSt_2a-sign(qSt_2a)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
   end
    if qSt_2b > q_max(2,1) |  qSt_2b < q_min(2,1)
        qSt_2b=qSt_2b-sign(qSt_2b)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
    end
    
    qSt_1a=p_vek(3,1)-a2*sin(qSt_2a);
    qSt_1b=p_vek(3,1)-a2*sin(qSt_2b);   
    
    % Gltigkeit beider Lsungen a,b berprfen
    % Prfung, ob im Bewegungsbereich, Zuordnung der Lsungen zum
    % Konfigurationsraum AR1 oder AR2"
    
    
    loes_St_AR1=0; % Vorbesetzung
    loes_St_AR2=0; 
    
    if (qSt_1a <= q_max(1,1) & qSt_1a >= q_min(1,1)) & ...
         (qSt_2a <= q_max(2,1) & qSt_2a >= q_min(2,1))   
        
        if qSt_2a >=0 & qSt_2a <= pi
            qSt_2_AR1=qSt_2a;
            qSt_1_AR1=qSt_1a;
            loes_St_AR1=1;
        else
            qSt_2_AR2=qSt_2a;
            qSt_1_AR2=qSt_1a;
            loes_St_AR2=1;
        end
    end    ;
    if (qSt_1b <= q_max(1,1) & qSt_1b >= q_min(1,1)) & ...
         (qSt_2b <= q_max(2,1) & qSt_2b >= q_min(2,1))   
            
           if qSt_2b >=0 & qSt_2b <= pi
            qSt_2_AR1=qSt_2b;
            qSt_1_AR1=qSt_1b;
            loes_St_AR1=1;
        else
            qSt_2_AR2=qSt_2b;
            qSt_1_AR2=qSt_1b;
            loes_St_AR2=1;
        end
    end    
       
    %qSt_1_AR1
    %qSt_2_AR1
    %qSt_1_AR2
    %qSt_2_AR2
    
   
       %xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        % Lsungen a,b fr den Zielpunkt
        qZ_2a=acos(-p_vek(1,1)/a2); % liefert Werte zwischen 0 und 180
        qZ_2b=-qZ_2a;
    
      
   if qZ_2a > q_max(2,1) |  qZ_2a < q_min(2,1)
        qZ_2a=qZ_2a-sign(qZ_2a)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
   end
    if qZ_2b > q_max(2,1) |  qZ_2b < q_min(2,1)
        qZ_2b=qZ_2b-sign(qZ_2b)*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
    end
    
    qZ_1a=p_vek(3,1)-a2*sin(qZ_2a);
    qZ_1b=p_vek(3,1)-a2*sin(qZ_2b);
    
       
     % Prfung, ob im Bewegungsbereich, Zuordnung der Lsungen zum
    % Konfigurationsraum AR1 oder AR2"
    loes_Z_AR1=0; % Vorbesetzung
    loes_Z_AR2=0;
    
    if (qZ_1a <= q_max(1,1) & qZ_1a >= q_min(1,1)) & ...
         (qZ_2a <= q_max(2,1) & qZ_2a >= q_min(2,1)) 
     if qZ_2a >=0 & qZ_2a <= pi
            loes_Z_AR1=1;
        else
            loes_Z_AR2=1;
        end
    end    ;
        
        
    if (qZ_1b <= q_max(1,1) & qZ_1b >= q_min(1,1)) & ...
         (qZ_2b <= q_max(2,1) & qZ_2b >= q_min(2,1))   
      if qZ_2b >=0 & qZ_2b <= pi
            loes_Z_AR1=1;
        else
            loes_Z_AR2=1;
        end
    end    ;      
      
      
    % hier wird fr die ganze Bahn der startvektor q_vek(:,1) festgelegt
        
       %xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
       % Durch Festsetzunng von q_vek(:,1) wird die weitere Lsung der
       % Rckwrtstransformation an den festgelegten Konfigurationsraum angepasst
       % Durch die Prfung (Pruef_Bahn) bei der Eingabe muss wenigstens eine Lsung im
       % Start oder Zielbereich mglich sein
       %
       % Flle:
       % 1. Start: AR1: ja    AR2: ja     Ziel: AR1: ja    AR2: ja     -->
       % Start mit AR1
       % 2. Start: AR1: ja    AR2: ja     Ziel: AR1: ja    AR2: nein -->
       % Start mit AR1
       % 3. Start: AR1: ja    AR2: ja     Ziel: AR1: nein  AR2: ja  -->
       % Start mit AR2
       % 4. Start: AR1: ja    AR2: nein   Ziel: AR1: ja    AR2: ja   -->
       % Start mit AR1
       % 5. Start: AR1: ja    AR2: nein   Ziel: AR1: ja    AR2: nein  -->
       % Start mit AR1
       % 6. Start: AR1: ja    AR2: nein   Ziel: AR1: nein  AR2: ja --->
       % Start mit AR1 Umorientierung!!!!!!!!!!!
       % 7. Start: AR1: nein  AR2: ja     Ziel: AR1: ja    AR2: ja --->
       % Start mit AR2
       % 8. Start: AR1: nein  AR2: ja     Ziel: AR1: ja    AR2: nein --->
       % Start mit AR2 Umorientierung!!!!!!!!!!!
       % 9. Start: AR1: nein  AR2: ja     Ziel: AR1: nein  AR2: ja --->
       % Start mit AR2 
       
       if loes_St_AR1==1 & loes_Z_AR1==1  %1,2,4,5
           q_vek(1,1)= qSt_1_AR1;
           q_vek(2,1)= qSt_2_AR1;  % es wird AR 1 genommen (Vorzug)
       end
       
       if loes_St_AR1==1 & loes_Z_AR1==0  % Fall 3
           % loes_Z_AR2 muss hier 1 sein, da mind eine Lsung im AR
           if loes_St_AR2==1 % 3
              q_vek(1,1)= qSt_1_AR2;
              q_vek(2,1)= qSt_2_AR2;
           else
              q_vek(1,1)= qSt_1_AR1;  % Fall 6
              q_vek(2,1)=  qSt_2_AR1; 
              disp('vermutlich muss umkonfiguriert werden!')%6
           end
       end
        %if loes_St_AR1==0 & loes_Z_AR2==1 % hier auch loes_St_AR2=1 
         %  q_vek(1,1)= qSt_1_AR2;
         %  q_vek(2,1)= qSt_2_AR2;  %7,9
       % elseif loes_St_AR1==0 & loes_Z_AR2==0 % hier auch loes_St_AR2=1 und loes_Z_AR1 = 1
        %   q_vek(1,1)= qSt_1_AR2;
        %   q_vek(2,1)= qSt_2_AR2;  %8 
        %   disp('vermutlich muss umkonfiguriert werden!')
        %else
        %   '   ' % no operation  
        %end
       if loes_St_AR1==0  % hier auch loes_St_AR2=1 
           q_vek(1,1)= qSt_1_AR2;
           q_vek(2,1)= qSt_2_AR2;  %7,8,9
          if loes_Z_AR2==0
              disp('es muss umkonfiguriert werden')
          end
       end
    end 
 %   'Loesung im Startpunkt (bzw. Zielpunkt) , Auswahl zwischen q_veka und q_vekb'

 
        %q_dist=abs(q_veka(:,1)-q_mitte); % absoluter Distanzvektor zur Mitte des Bewegungsbereiches
       % q_dist_norm=q_dist./q_bereich ;% q_dist wird hier auf die absoluten Werte des Bewegungsbereiches bezogen
       % ja=norm(q_dist_norm);
        
       % q_dist=abs(q_vekb(:,1)-q_mitte); % absolute Distanz zur Mitte des Bewegungsbereiches
       % q_dist_norm=q_dist./q_bereich;
       % jb=norm(q_dist_norm);
       % if ja<=jb
       % q_vek(1,1)=q_veka(1,1);
       % q_vek(2,1)=q_veka(2,1);
      %  else
       % q_vek(1,1)=q_vekb(1,1);
       % q_vek(2,1)=q_vekb(2,1);
       % end

   
  
   
   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hier beginnt die Hauptschleife
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%disp('vor Start der Hauptschleife, das erste Element von q_vek')
%'1. Spalte'
%q_vek(:,1)

%'das erste Element von p_vek'
%'1. Spalte'
%p_vek(:,1)


% 
% Beim Startpunkt und Zielpunkt ist anz_schritte=1 
%_______________________________________________________________________________________

for i1=2:anz_schritte
 
  
    q_veka(2,i1)=acos(-p_vek(1,i1)/a2);
    q_vekb(2,i1)=- q_veka(2,i1);
 
    % Es folgt die Abbildung des Winkels q2 in den numerisch angegebenen
    % Bewegungsbereich: bsp: q2=-230 --> q2b=130
    if q_veka(2,i1) > q_max(2,1) |  q_veka(2,i1)< q_min(2,1)
        q_veka(2,i1)=q_veka(2,i1)-sign(q_veka(2,i1))*2*pi; % Abbildung, wenn mglich, in die num. Angabe des Bewegungsbereich 
    end
       
     if q_vekb(2,i1) > q_max(2,1) |  q_vekb(2,i1)< q_min(2,1)
        q_vekb(2,i1)=q_vekb(2,i1)-sign(q_vekb(2,i1))*2*pi;
     end
    
    q_veka(1,i1)=p_vek(3,i1)-a2*sin(q_veka(2,i1));
    q_vekb(1,i1)=p_vek(3,i1)-a2*sin(q_vekb(2,i1));
   
    % Prfung, ob im Bewegungsbereich und Abstand zur alten Lsung
    if (q_veka(1,i1) <= q_max(1,1) & q_veka(1,i1) >= q_min(1,1)) & ...
         (q_veka(2,i1) <= q_max(2,1) & q_veka(2,i1) >= q_min(2,1))   
        loes_a=1;
        ja=norm(q_veka(:,i1)-q_vek(:,i1-1));
        %ja=norm(q_veka(:,i1)-q_mitte);
    else 
        loes_a=0;
        ja=123000000000;
    end
    if (q_vekb(1,i1) <= q_max(1,1) & q_vekb(1,i1) >= q_min(1,1)) & ...
         (q_vekb(2,i1) <= q_max(2,1) & q_vekb(2,i1) >= q_min(2,1))   
        loes_b=1;
        %q_alt=q_vek(:,i1-1)
        jb=norm(q_vekb(:,i1)-q_vek(:,i1-1));
        %jb=norm(q_vekb(:,i1)-q_mitte);
    else 
        loes_b=0;
        jb=12300000000000;
    end
    % Auswahl der Lsung
    if loes_a==1 & loes_b==1
        if ja<=jb
        q_vek(1,i1)=q_veka(1,i1);
        q_vek(2,i1)=q_veka(2,i1);
        else
        q_vek(1,i1)=q_vekb(1,i1);
        q_vek(2,i1)=q_vekb(2,i1);
        end
    end
    if loes_a==1 & loes_b==0
        q_vek(1,i1)=q_veka(1,i1);
        q_vek(2,i1)=q_veka(2,i1);
    elseif  loes_a==0 & loes_b==1 
        q_vek(1,i1)=q_vekb(1,i1);
        q_vek(2,i1)=q_vekb(2,i1);
    elseif loes_a==0 & loes_b==0
        ' keine Lsung!!!????'
        return
    end
    
  
 end



% Zur Berechnung von qp_vek wird die Jacobimatrix verwendet
% [xo_p; z0_p]=J*[q1_p;q2_p]
%J=[0 a2*cos(q2-90); 1 -a2*sin(q2-909];
% inv(J)= [tan(q2-90) 1 ; 1/(a2*cos(q2-90) 0]


qp_vek(1,1)=0;
qp_vek(2,1)=0;    % Geschwindigkeiten am Start sind 0

for i1=2:anz_schritte
       if abs(sin(q_vek(2,i1))) < eps_sing
      disp('singulre Stellung')
      if i1< anz_schritte
        qp_vek(1,i1)= (q_vek(1,i1+1)-q_vek(1,i1-1))/(2*T_Ipo);
        qp_vek(2,i1)= (q_vek(2,i1+1)-q_vek(2,i1-1))/(2*T_Ipo);
      else
        qp_vek(1,i1)=(q_vek(1,i1)-q_vek(1,i1-1))/T_Ipo;
        qp_vek(2,i1)=(q_vek(2,i1)-q_vek(2,i1-1))/T_Ipo;
      end
    else
        %disp('keine singulre Stelle')
        qp_vek(1,i1)=-1/tan(q_vek(2,i1)) * p_punkt_vek(1,i1) + p_punkt_vek(3,i1);
        qp_vek(2,i1)=1/(a2*q_vek(2,i1))* p_punkt_vek(1,i1);
        %qp_vek(2,i1)=1/(a2*cos(q_vek(2,i1)-pi/2))* p_punkt_vek(1,i1);       % diese Lsung bis 1.9.23
        %qp_vek(1,i1)=sin(q_vek(2,i1)-pi/2) * a2*qp_vek(2,i1) + p_punkt_vek(3,i1); % diese Lsung bis 1.2.23
    
       end

end

%test_singulaer(1,:)=abs(cos(q_vek(2,:)-pi/2))
%abs(cos(q_vek(2,:)-pi/2)) > eps_sing 
%qp_vek(1,:)=tan(q_vek(2,:)-pi/2) .* p_punkt_vek(1,:) + p_punkt_vek(3,:);
%qp_vek(2,:)=1./(a2*cos(q_vek(2,:)-pi/2)).* p_punkt_vek(1,:);

    

