function [s,vel,acc] = rampe(se,tb,tv,te,vm,am,dt)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%rampe.m
%RAMPE		Berechnung Bahnprofil Rampe (Strecke, Geschwindigkeit, Beschleunigung)
% 
% [s,vel,acc] = rampe(se,tb,tv,te,vm,am,dt)
%
%
% INPUT:	
%	se	[	se1		Lnge einer Bahn (Winkel, kartesisch, Orientierung)
%			se2
%			... ]
%
%	tb	Beschleunigungszeit in Sekunden
%	tv	Bremspunkt
%	te	Bahndauer in Sekunden
%	vm	Geschwindigkeit
%	bm	Beschleunigung
%	dt	Interpolationstakt
% 
% OUTPUT:
%	s	[	s1(t)		Strecke (t)
%			s2(t)
%			... ]
%
%	vel	[	v1(t)		Geschwindigkeit (t)
%			v2(t)
%			... ]
%
%	acc	[	a1(t)		Beschleunigung (t)
%			a2(t)
%			... ]
%
% globale Variablen:
%	keine
%		
% aufgerufene Funktionen:
%	keine
% erstellt von J. Dimmerling und J. Rotter
% nderungen: April 2001 von W. Weber, Vermeidung numerischer Ungenauigkeiten
% Letzte nderung August 2011, Weg beginnt jetzt bei s=0 nicht bei s=0.5*bm*T_ipo^2 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



rows = size(se,1);

temax= max(te);			% nur fr PTP asynchron
tast  = round(temax/dt)+1;  	% Anzahl der Spalten der Gelenkwinkelmatrix und des Zeitvektors

st = zeros(rows,1);		% vordimensionieren
s  = zeros(rows,tast);		% ==> Geschwindigkeitsvorteil

vt   = zeros(rows,1);
at   = zeros(rows,1);
vel  = zeros(rows,tast);
acc  = zeros(rows,tast);
zero = zeros(rows,1);

s (:,1)  = 0;
  vel(:,2) =0;
  acc(:,2) = am;
	
for j=2:1:tast,

  t = j *dt;
  
  i= find( 0  <=t  &  t <= tb );
  
    st(i) = (.5*am(i))*t^2;
    vt(i) = am(i)*t;
    at(i) = +am(i);
  
  i= find( tb < t  &  t <= tv );
  
    st(i) = vm(i) *t - (vm(i).^2)./(2 *am(i));
    vt(i) = vm(i);
    at(i) = zero(i);
  
  i= find( tv < t  &  t <= te );
  
    st(i) = se(i) - (.5*am(i)).*((te(i)-t).^2);
    vt(i) = am(i).*(te(i)-t);
    at(i) = -am(i);
  
  i= find( te < t );
  
    st(i) = se(i);
    vt(i) = zero(i);
    at(i) = zero(i);
	
  s (:,j)  = st;
  vel(:,j) = vt;
  acc(:,j) = at;

end;


